@@ -3,7 +3,7 @@ import uuid
 from typing import Any, Sequence
 
 
-def escape(value: Any):
+def escape(value: Any) -> str:
     if isinstance(value, str):
         return f"E{repr(value)}"
     elif isinstance(value, float) or isinstance(value, int):
@@ -17,3 +17,7 @@ def escape(value: Any):
     elif isinstance(value, Sequence):
         args = ", ".join(escape(x) for x in value)
         return f"ARRAY[{args}]"
+    elif value is None:
+        return "NULL"
+    else:
+        raise TypeError(f"Can't escape type {type(value)}")
